"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RushStackConfig = exports.loadHeftConfig = void 0;
const fs_1 = require("fs");
const module_1 = require("module");
const path_1 = require("path");
const json_extends_loader_1 = require("@idlebox/json-extends-loader");
const tsconfig_loader_1 = require("@idlebox/tsconfig-loader");
const rig_package_1 = require("@rushstack/rig-package");
const cache = new WeakMap();
const isModuleResolutionError = (ex) => typeof ex === 'object' &&
    !!ex &&
    'code' in ex &&
    (ex.code === 'MODULE_NOT_FOUND' || ex.code === 'ERR_MODULE_NOT_FOUND');
function loadHeftConfig(heftConfiguration) {
    if (!cache.has(heftConfiguration)) {
        cache.set(heftConfiguration, new RushStackConfig(heftConfiguration.buildFolder, heftConfiguration.rigConfig));
    }
    return cache.get(heftConfiguration);
}
exports.loadHeftConfig = loadHeftConfig;
class RushStackConfig {
    warning;
    typescriptConfig;
    tsconfigConfig;
    apiExtractorConfig;
    rigConfig;
    localRequire;
    rigRequire;
    localRequirePath;
    rigRequirePath;
    projectFolder;
    /**
     *
     * @param projectFolder absolute path of directory where the package.json is in
     */
    constructor(projectFolder, rigConfig, warning = console.warn) {
        this.warning = warning;
        this.rigConfig = rigConfig || rig_package_1.RigConfig.loadForProjectFolder({ projectFolderPath: projectFolder });
        if (this.rigConfig.rigFound) {
            this.rigRequirePath = this.rigConfig.getResolvedProfileFolder();
            this.rigRequire = (0, module_1.createRequire)(this.rigRequirePath);
        }
        this.projectFolder = this.rigConfig.projectFolderPath;
        this.localRequirePath = this.projectFolder;
        this.localRequire = (0, module_1.createRequire)(this.localRequirePath + '/package.json');
        this.resolve = this.resolve.bind(this);
        this.require = this.require.bind(this);
        Object.assign(this.require, { resolve: this.resolve });
    }
    resolveConfigFile(name) {
        let path = (0, path_1.resolve)(this.projectFolder, 'config', name);
        if ((0, fs_1.existsSync)(path)) {
            return path;
        }
        if (!(0, fs_1.existsSync)((0, path_1.resolve)(this.projectFolder, 'package.json'))) {
            throw new Error(`can not resolve ${name}. another error during resolve: package.json did not exists in projectFolder(${this.projectFolder})`);
        }
        return this.loadRig('typescript.json');
    }
    loadRig(file) {
        if (!this.rigConfig.rigFound) {
            return undefined;
        }
        const cfgFile = (0, path_1.resolve)(this.rigConfig.getResolvedProfileFolder(), file);
        if ((0, fs_1.existsSync)(cfgFile)) {
            return cfgFile;
        }
        return undefined;
    }
    require(packageName) {
        return require(this.resolve(packageName));
    }
    resolve(packageName) {
        let e1;
        try {
            return this.localRequire.resolve(packageName);
        }
        catch (e) {
            if (!isModuleResolutionError(e)) {
                throw e;
            }
            e1 = e;
        }
        if (this.rigRequire) {
            try {
                return this.rigRequire.resolve(packageName);
            }
            catch (e) {
                if (!isModuleResolutionError(e)) {
                    throw e;
                }
            }
        }
        const e = Object.create(e1.constructor.prototype);
        let message = `Cannot find module '${packageName}'\nRequire stack:\n- ${this.localRequirePath}`;
        if (this.rigRequirePath) {
            message += '\n- ' + this.rigRequirePath;
        }
        throw Object.assign(e, { code: 'MODULE_NOT_FOUND', message, stack: e1.stack });
    }
    apiExtractor() {
        if (!this.apiExtractorConfig) {
            const file = this.resolveConfigFile('api-extractor.json');
            if (file) {
                const apiExtractor = this.require('@microsoft/api-extractor');
                this.apiExtractorConfig = wrapApiExtractorConfig(this.projectFolder, apiExtractor.ExtractorConfig.loadFile(file));
            }
        }
        return this.apiExtractorConfig;
    }
    typescript() {
        if (!this.typescriptConfig) {
            let cfgFile = this.resolveConfigFile('typescript.json');
            let cfg;
            if (cfgFile) {
                cfg = (0, json_extends_loader_1.loadInheritedJson)(cfgFile, {
                    cwd: this.projectFolder,
                    readJsonFile: (0, json_extends_loader_1.createDynamicReader)(resolveProjectFolder),
                });
            }
            else {
                this.warning('missing config/typescript.json (searched rig package), using internal resolver.');
                cfg = {
                    project: (0, path_1.resolve)(this.projectFolder, 'tsconfig.json'),
                    maxWriteParallelism: undefined,
                };
            }
            this.typescriptConfig = cfg;
        }
        return this.typescriptConfig;
    }
    tsconfigPath() {
        const cfg = this.typescript().project;
        if (cfg)
            return cfg;
        this.warning(`missing "project" field in "typescript.json"`);
        return (0, path_1.resolve)(this.projectFolder, 'tsconfig.json');
    }
    tsconfig() {
        if (!this.tsconfigConfig) {
            const proj = this.tsconfigPath();
            this.tsconfigConfig = (0, tsconfig_loader_1.loadTsConfigJsonFile)(proj, this.require('typescript'));
        }
        return this.tsconfigConfig;
    }
}
exports.RushStackConfig = RushStackConfig;
function resolveProjectFolder(filePath, data) {
    if (data.project) {
        data.project = (0, path_1.resolve)(filePath, '../..', data.project);
    }
}
function extendStrings(obj, i, map) {
    if (!obj || typeof obj[i] !== 'string')
        return;
    let c = obj[i];
    for (const [from, to] of Object.entries(map)) {
        c = c.replaceAll(from, to);
    }
    obj[i] = c;
}
function wrapApiExtractorConfig(packagePath, config) {
    const packageName = require((0, path_1.resolve)(packagePath, 'package.json')).name;
    const unscopedPackageName = (0, path_1.basename)(packageName);
    const pu = {
        '<packageName>': packageName,
        '<unscopedPackageName>': unscopedPackageName,
    };
    const ppu = {
        '<projectFolder>': config.projectFolder,
        ...pu,
    };
    extendStrings(config, 'mainEntryPointFilePath', ppu);
    extendStrings(config.compiler, 'tsconfigFilePath', ppu);
    extendStrings(config.apiReport, 'reportFileName', pu);
    extendStrings(config.apiReport, 'reportFolder', ppu);
    extendStrings(config.apiReport, 'reportTempFolder', ppu);
    extendStrings(config.docModel, 'apiJsonFilePath', ppu);
    extendStrings(config.dtsRollup, 'untrimmedFilePath', ppu);
    extendStrings(config.dtsRollup, 'alphaTrimmedFilePath', ppu);
    extendStrings(config.dtsRollup, 'betaTrimmedFilePath', ppu);
    extendStrings(config.dtsRollup, 'publicTrimmedFilePath', ppu);
    extendStrings(config.tsdocMetadata, 'tsdocMetadataFilePath', ppu);
    return config;
}
//# sourceMappingURL=api.js.map